"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CobblerRoute = exports.InitCobblerRoute = void 0;
const gdut_json_1 = require("gdut-json");
const pluralize_1 = require("pluralize");
const cobbler_1 = require("./cobbler");
const cobbler_handler_1 = require("./cobbler-handler");
function InitCobblerRoute(app, path, apiRoot = '/api/', handler = new cobbler_handler_1.CobblerHandler(), cache = true) {
    const tables = gdut_json_1.ReadJson(`${path}/tables.json`);
    handler.beforeInit(tables);
    const cobbler = new cobbler_1.Cobbler(`${path}/data`, tables, handler, cache);
    handler.afterInit(cobbler);
    app.all(new RegExp(`${apiRoot}.*$`, 'i'), function (req, res) {
        const pathInfo = req.url.replace(apiRoot, '').split('/');
        console.log(`${req.method} API: ${pathInfo.length}`, pathInfo);
        // console.log({ apiRoot, reqUrl: req.url, pathInfo });
        if (handler.beforeRoute(req.method, pathInfo, req, res)) {
            CobblerRoute(cobbler, req.method, pathInfo, req, res, handler);
            handler.afterRoute(req.method, pathInfo, req, res);
        }
    });
}
exports.InitCobblerRoute = InitCobblerRoute;
function CobblerRoute(cobbler, method, pathInfo, req, res, handler) {
    try {
        const table = CobblerParseTable(cobbler, pathInfo);
        let id = pathInfo.length > 1 ? parseInt(pathInfo[1]) : null;
        if (id != null && isNaN(id)) {
            id = null;
        }
        if (method === 'GET') {
            CobblerReadRoute(cobbler, id, table, req, res);
        }
        else if (['POST', 'PATCH', 'PUT'].includes(method)) {
            CobblerSaveRoute(cobbler, req, id, table, res);
        }
        else if (method === 'DELETE') {
            if (id !== null) {
                const record = cobbler.remove(table, id);
                res.json(record);
            }
            else {
                throw new Error(`ID_NOT_FOUND; Remove failed id not found! 😭`);
            }
        }
        // curl -X DELETE http://localhost:3001/users/1
    }
    catch (err) {
        console.error('CobblerRoute Error', err);
        res.json({ error: err.toString() });
    }
}
exports.CobblerRoute = CobblerRoute;
function CobblerParseTable(cobbler, pathInfo) {
    let name = pathInfo[0].replace(/\?.*/, '');
    if (name === '') {
        return name;
    }
    else {
        const tables = cobbler.getTables();
        if (tables.find((el) => el.tableName === name)) {
            return name;
        }
        else {
            const pluralName = pluralize_1.plural(name);
            if (tables.find((el) => el.tableName === pluralName)) {
                return pluralName;
            }
        }
    }
    return name;
}
function CobblerSaveRoute(cobbler, req, id, table, res) {
    const record = req.body;
    if (Array.isArray(record)) {
        const saved = cobbler.saveAll(table, record);
        res.json(saved);
    }
    else {
        return CobblerSaveOne(cobbler, record, id, table, res);
    }
}
function CobblerSaveOne(cobbler, record, id, table, res) {
    console.log(`MODIFY/ NEW, id: ${id}`, record);
    if (id !== null) {
        record.id = id;
    }
    const saved = cobbler.save(table, record);
    console.log('saved', saved);
    res.json(saved);
}
function CobblerReadRoute(cobbler, id, table, req, res) {
    if (id !== null) {
        const record = cobbler.findById(table, id);
        res.json(record);
    }
    else if (table === '') {
        res.json(cobbler.getTables().map((table) => {
            return Object.assign({ apiUrl: `${req.url}${table.tableName}` }, table);
        }));
    }
    else {
        console.log('req.query', req.query);
        if (req.query['size']) {
            const size = parseInt(req.query['size']);
            let page = parseInt(req.query['page']);
            let sort = req.query['sort'] ? req.query['sort'] : 'id';
            let sortDir = 'ASC';
            if (isNaN(page)) {
                page = 0;
            }
            const sortTags = sort.split(',');
            if (sortTags.length > 1) {
                sort = sortTags[0];
                sortDir = sortTags[1] === 'DESC' ? 'DESC' : 'ASC';
            }
            console.log('findPage', { table, page, size, sort, sortDir });
            const records = cobbler.findPage(table, page, size, sort, sortDir, req.query);
            res.json(records);
        }
        else {
            const records = cobbler.findAll(table, req.query);
            res.json(records);
        }
    }
}
