"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Serve = void 0;
const express_1 = __importDefault(require("express"));
const gdut_io_1 = require("gdut-io");
const gdut_json_1 = require("gdut-json");
const basic_auth_1 = require("../basic-auth/basic-auth");
// import basicAuth from 'express-basic-auth';
const cobbler_api_1 = require("./cobbler-api");
const cobbler_handler_1 = require("./cobbler-handler");
function Serve(handler = new cobbler_handler_1.CobblerHandler()) {
    var _a;
    let config = { port: 7117, path: './data', url: '/api/', cache: true };
    if (handler.config) {
        config = handler.config;
    }
    else if (gdut_io_1.FileExists('./cobbler.json')) {
        config = gdut_json_1.ReadJson('./cobbler.json');
    }
    const app = express_1.default();
    if (config.basicAuth) {
        app.use(basic_auth_1.basicAuth({
            users: config.basicAuth.users,
            realm: function (req) {
                console.log('req.headers', JSON.stringify(req.headers));
                if (req.headers['x-requested-with']) {
                    return 'NO_REALM';
                }
                else {
                    return '';
                }
            },
            challenge: true,
        }));
    }
    handler.initExpress(app);
    const PORT = config.port;
    console.log('Server TypeScript');
    // Needed for parsing Request Body to JSON Object:
    app.use(express_1.default.json());
    app.get('/helloWorld', (req, res) => {
        res.send('Hello World!');
    });
    cobbler_api_1.InitCobblerRoute(app, config.path, config.url, handler, (_a = config.cache) !== null && _a !== void 0 ? _a : true);
    app.listen(PORT, () => console.log(`Server listening on port: ${PORT}`));
}
exports.Serve = Serve;
