import { CobblerHandler } from './cobbler-handler';
export declare type Record = {
    id: number;
    logicRemove: boolean;
    [key: string]: any;
};
export declare type ColumnConfig = {
    type: string;
    needed: boolean;
    default: any;
    reference?: string;
    unique: boolean;
    owner: true;
};
export declare type TableConfig = {
    [key: string]: ColumnConfig;
};
export declare type TablesConfig = {
    [key: string]: TableConfig;
};
export declare function BaseCobbler(path?: string): Cobbler;
export declare class Cobbler {
    private dataPath;
    private tables;
    private handler;
    private cache;
    tableCache: {
        [key: string]: Record[];
    };
    constructor(dataPath: string, tables: TablesConfig, handler?: CobblerHandler, cache?: boolean);
    getTables(): {
        tableName: string;
    }[];
    saveAll(name: string, records: Record[]): any[];
    save(name: string, record: Record): any;
    private callUpdate;
    private callInsert;
    remove(name: string, id: number): Record;
    findById(name: string, id: number): {
        [x: string]: any;
        id: number;
        logicRemove: boolean;
    } | null;
    findAll(name: string, parms?: any): {
        [x: string]: any;
        id: number;
        logicRemove: boolean;
    }[];
    columnFilter(name: string, parms: any, record: Record): unknown;
    invokeColumnFilter(name: string, column: ColumnConfig, parms: any, record: Record): boolean;
    findPage(name: string, page: number, size: number, sortName: string, sortDir: 'ASC' | 'DESC', parms: any): {
        count: number;
        items: {
            [x: string]: any;
            id: number;
            logicRemove: boolean;
        }[];
    };
    private transformResult;
    private transformRecord;
    private collectOwnedReferences;
    private prepareOwnedReferences;
    private getReferenceColumnName;
    private saveOwnedReferences;
    private update;
    private insert;
    private validate;
    private checkUnique;
    private checkColumnType;
    private checkReference;
    private locateReference;
    private locate;
    private readDataFile;
    private writeDataFile;
}
